pub(crate) struct ManifestContext {
pub(crate) manifest_manager: RwLock<RegionManifestManager>,
state: AtomicCell<RegionRoleState>,
staging_partition_info: Mutex<Option<StagingPartitionInfo>>,
}Expand description
Context to update the region manifest.
Fields
manifest_manager: RwLock<RegionManifestManager>Manager to maintain manifest for this region.
state: AtomicCell<RegionRoleState>The state of the region. The region checks the state before updating manifest.
staging_partition_info: Mutex<Option<StagingPartitionInfo>>Partition info of the region in staging mode.
During the staging mode, the region metadata in VersionControlRef is not updated,
so we need to store the partition info separately.
Implementations
Sourceimpl ManifestContext
impl ManifestContext
pub(crate) fn new( manager: RegionManifestManager, state: RegionRoleState, ) -> Self
pub(crate) fn staging_partition_info(&self) -> Option<StagingPartitionInfo>
pub(crate) fn set_staging_partition_info( &self, staging_partition_info: StagingPartitionInfo, )
fn clear_staging_partition_info(&self)
pub(crate) fn exit_staging( &self, region_id: RegionId, next_state: RegionRoleState, ) -> Result<()>
pub(crate) async fn manifest_version(&self) -> ManifestVersion
pub(crate) async fn has_update(&self) -> Result<bool>
Sourcepub(crate) fn current_state(&self) -> RegionRoleState
pub(crate) fn current_state(&self) -> RegionRoleState
Returns the current region role state.
Sourcepub(crate) async fn install_manifest_to(
&self,
version: ManifestVersion,
) -> Result<Arc<RegionManifest>>
pub(crate) async fn install_manifest_to( &self, version: ManifestVersion, ) -> Result<Arc<RegionManifest>>
Installs the manifest changes from the current version to the target version (inclusive).
Returns installed RegionManifest. Note: This function is not guaranteed to install the target version strictly. The installed version may be greater than the target version.
Sourcepub(crate) async fn update_manifest(
&self,
expect_state: RegionLeaderState,
action_list: RegionMetaActionList,
is_staging: bool,
) -> Result<ManifestVersion>
pub(crate) async fn update_manifest( &self, expect_state: RegionLeaderState, action_list: RegionMetaActionList, is_staging: bool, ) -> Result<ManifestVersion>
Updates the manifest if current state is expect_state.
Sourcepub(crate) fn set_role(&self, next_role: RegionRole, region_id: RegionId)
pub(crate) fn set_role(&self, next_role: RegionRole, region_id: RegionId)
Sets the [RegionRole].
+---------------------+
| Staging Leader |
+----------+----------+
|
v
+----------+ +------+-------+ +-------------+
| Follower | <-> | Leader | <-> | Downgrading |
+-----+----+ +------+-------+ +------+------+
^ ^ |
+-----------------+--------------------+
State Transitions
From Follower:
Follower -> Leader
From Leader:
Leader -> FollowerLeader -> Downgrading Leader
From Staging Leader:
Staging Leader -> LeaderStaging Leader -> FollowerStaging Leader -> Downgrading Leader
From Downgrading Leader:
Downgrading Leader -> LeaderDowngrading Leader -> Follower
Sourcepub(crate) async fn manifest(&self) -> Arc<RegionManifest>
pub(crate) async fn manifest(&self) -> Arc<RegionManifest>
Returns the normal manifest of the region.
Sourcepub(crate) async fn staging_manifest(&self) -> Option<Arc<RegionManifest>>
pub(crate) async fn staging_manifest(&self) -> Option<Arc<RegionManifest>>
Returns the staging manifest of the region.